unit Helios_ROQ_Engine;

interface

uses qne_RoQ;

Function IsRoqActive : Boolean;

Procedure LoadRoq(Filename : String);
Procedure ReadNextRoqFrame;
Procedure RenderRoq(Cinematic : PCinematic);
Procedure StopRoq;

Var
Cinematic : PCinematic;

Const
  AUDIO_BUFFER_SAMPLE_COUNT = 65536;

implementation

uses {Helios_Global,}Math3d,OpenGL15,Helios_Timer,SysUtils;

Function IsRoqActive : Boolean;
begin

Result := Assigned(Cinematic);

end;

Procedure LoadRoq(Filename : String);
var
dest : Array of Byte;
begin

//LockStream := True;
  try
    Cinematic := RoQ_Open(Filename, False, AUDIO_BUFFER_SAMPLE_COUNT);
    if not Assigned(Cinematic) then
    begin
//      AddToConsole('An error has occured while opening RoQ file ' + Filename);
//      LockStream := False;
      Exit;
    end;

    Cinematic.StartTime := 0;

//    LblInfo.Caption := Format('%dx%d, %d FPS',
//                              [Cinematic.Width, Cinematic.Height, Cinematic.FPS]);

    // Create a texture we can render to
    glGenTextures(1, @Cinematic.Texture);
    glEnable(GL_TEXTURE_2D);
    glBindTexture(GL_TEXTURE_2D, Cinematic.Texture);

    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);

    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);

    SetLength(dest, Cinematic.Width*Cinematic.Height*3);
    glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, Cinematic.Width, Cinematic.Height, 0, GL_RGB, GL_BYTE, dest);

    ReadNextRoqFrame;

//    AudioOffset := 0;
{    fmt.freq := 22050;
    fmt.format := AUDIO_S16;
    fmt.channels := 2;
    fmt.samples := SDL_BUFFER_SAMPLE_COUNT;
    fmt.callback := sdl_mixaudio;
    fmt.userdata := Cinematic;

    SDL_CloseAudio;
    SDL_OpenAudio(@fmt, nil);
    SDL_PauseAudio(0);  }
  finally
  //  LockStream := False;
  end;
end;

Procedure ReadNextRoqFrame;
begin
if not RoQ_ReadNextFrame(Cinematic) then
  begin
  RoQ_Restart(Cinematic);
  ReadNextRoqFrame;
 //   AudioChannels := 0;
 //   FillChar(AudioBuffer[0], AUDIO_BUFFER_SAMPLE_COUNT * SizeOf(SmallInt), 0);
 // RoQ_Close(Cinematic);
 //   SDL_CloseAudio;
  end
  else
  begin
  //  AudioChannels := Cinematic.AudioChannels;
  //  Move(Cinematic.AudioBuffer^, AudioBuffer[0],
  //       AUDIO_BUFFER_SAMPLE_COUNT * SizeOf(SmallInt));
  end;
end;

Procedure RenderRoq(Cinematic : PCinematic);
var
width,height,x,y,c,cur_frame : Integer;
src: PByteArray;
dest : Array of Byte;
begin

    If Cinematic.State = 1 then // If playing
    Cinematic.StartTime := Cinematic.StartTime + gTimer.FrameTime;
    cur_frame := Trunc((Cinematic.StartTime) * Cinematic.FPS);

    if cur_frame <= Cinematic.CurrentFrame-1 then
    Exit;

    Width := Cinematic.Width;
    Height := Cinematic.Height;
    SetLength(dest, Width*Height*3);

    c := 0;
    for y := 0 to Height-1 do
    begin
      src := @Cinematic.FrameBuffer[0, (Height-y-1) * Width * 3];

      for x := 0 to Width-1 do
      begin
        dest[c*3+0] := src[x*3+0] div 2;
        dest[c*3+1] := src[x*3+1] div 2;
        dest[c*3+2] := src[x*3+2] div 2;

        inc(c);
      end;
    end;

    glEnable(GL_TEXTURE_2D);
    glBindTexture(GL_TEXTURE_2D,Cinematic.Texture);

    glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, Width, Height, 0, GL_RGB, GL_BYTE, dest);

    If Cinematic.State = 1 then
    ReadNextRoqFrame;
end;

Procedure StopRoq;
begin

RoQ_Restart(Cinematic);
ReadNextRoqFrame;

end;

end.
